/* $Id: sortprog.c,v 1.3 2004/01/21 20:49:43 dw Exp $ */
/* Author Damian Weber http://www-crypto.htw-saarland.de 
 * License: http://www.gnu.org/licenses/gpl-2.0.html
 */

#include <stdio.h>
#include <stdlib.h>

#include "sorting.h"

#undef DEBUG_Q
#undef SAME_SEQUENCE
#undef USE_SELECTION_SORT
#undef USE_SHELL_SORT
#undef USE_INSERTION_SORT

/* usage: sortprog <#elements> */

/* check, whether sequence is sorted */
/* return 1 if it is
 * else   0
 */

int check(int n, int *a)
{
	int j;

	for (j=0;j<n-1;j++)
		if (a[j]>a[j+1]) 
		{

			printf("\na[%d]=%d, a[%d]=%d\n",j,a[j],j+1,a[j+1]);
			return 0;
		}
	return 1;
}


void output(int n, int *a)
{
	int j;

	printf("------------------\n");
	for (j=0;j<n;j++)
	{
		if (j%10==0) printf("\n");
		printf("%8d",a[j]);
	}
	printf("\n");
	printf("------------------\n");
}

int main(int argc, char **argv)
{
	int t0; /* base time for procedure start */
	int t1; /* end  time for procedure start */
	int n;
	int *a; /* array containing the input */
	int rc; /* return code */
#ifdef SAME_SEQUENCE
	int *b;
#endif

	rc=0;
	if (argc<=1) return 1;
	n=atoi(argv[1]);
	a=(int *) malloc(n*sizeof(int));
#ifdef SAME_SEQUENCE
	b=(int *) malloc(n*sizeof(int));
	generate(0,n-1,b);
#endif



#ifdef DEBUG
	output(n,b);
#endif

	/* -------------- selection sort ------------------- */
#ifdef USE_SELECTION_SORT
#ifdef SAME_SEQUENCE
	memcpy(a,b,n*sizeof(int));
#else
	generate(0,n-1,a);
#endif
	t0=process_time();
	selection_sort(n,a);
	t1=process_time();
	printf("selection sort: %d hsec\n",t1-t0);
#ifdef DEBUG
	output(n,a);
#endif
	if (!check(n,a))
	{
		printf("error in sort program\n");
		rc=1;
	}

#endif

	/* -------------- merge     sort ------------------- */
#ifdef SAME_SEQUENCE
	memcpy(a,b,n*sizeof(int));
#else
	generate(0,n-1,a);
#endif
	t0=process_time();
	merge_sort(n,a);
	t1=process_time();
	printf("merge sort: %d hsec\n",t1-t0);
#ifdef DEBUG
	output(n,a);
#endif
	if (!check(n,a))
	{
		printf("error in sort program\n");
		rc=1;
	}



	/* -------------- heap      sort ------------------- */
#ifdef SAME_SEQUENCE
	memcpy(a,b,n*sizeof(int));
#else
	generate(0,n-1,a);
#endif
	t0=process_time();
	heap_sort(n,a);
	t1=process_time();
	printf("heap sort: %d hsec\n",t1-t0);
#ifdef DEBUG
	output(n,a);
#endif
	if (!check(n,a))
	{
		printf("error in sort program\n");
		rc=1;
	}


	/* -------------- ternary heap sort ------------------- */
#ifdef SAME_SEQUENCE
	memcpy(a,b,n*sizeof(int));
#else
	generate(0,n-1,a);
#endif
	t0=process_time();
	heap3_sort(n,a);
	t1=process_time();
	printf("heap3 sort: %d hsec\n",t1-t0);
#ifdef DEBUG
	output(n,a);
#endif
	if (!check(n,a))
	{
		printf("error in sort program\n");
		rc=1;
	}


	/* -------------- quick     sort ------------------- */
#ifdef SAME_SEQUENCE
	memcpy(a,b,n*sizeof(int));
#else
	generate(0,n-1,a);
#endif

#if 0
	output(n,a);
#endif
	t0=process_time();
	quick_sort(n,a);
	t1=process_time();
	printf("quick sort: %d hsec\n",t1-t0);
#if 0
	output(n,a);
#endif
	if (!check(n,a))
	{
		printf("error in sort program\n");
		rc=1;
	}

#ifdef USE_INSERTION_SORT
	/* -------------- insertion sort ------------------- */
#ifdef SAME_SEQUENCE
	memcpy(a,b,n*sizeof(int));
#else
	generate(0,n-1,a);
#endif
	t0=process_time();
	insertion_sort(n,a);
	t1=process_time();
	printf("insertion sort: %d hsec\n",t1-t0);
#ifdef DEBUG_Q
	output(n,a);
#endif
	if (!check(n,a))
	{
		printf("error in sort program\n");
		rc=1;
	}

#endif

#ifdef USE_SHELL_SORT
	/* -------------- shell sort ------------------- */
#ifdef SAME_SEQUENCE
	memcpy(a,b,n*sizeof(int));
#else
	generate(0,n-1,a);
#endif
	t0=process_time();
	shell_sort(n,a);
	t1=process_time();
	printf("shell sort: %d hsec\n",t1-t0);
#if 0
	output(n,a);
#endif
	if (!check(n,a))
	{
		printf("error in sort program\n");
		rc=1;
	}
#endif
	/* --------------                ------------------- */

	return rc;
}

